#include "Global.H"
#include "Graph.H"
#include "GameObj.H"
#include "Worker.H"
#include "Mineral.H"
#include <cassert>
#include <set>
int Graph::getVertex(Loc loc){
		if (coord2v.count(loc)==0){		
			coord2v[loc]=num_v;
			v2coord[num_v]=loc;
			num_v++;
		}
		return coord2v[loc];
}

bool Graph::testpath(line_seg a, sint4 d){
		pun p1(a.l1.x,a.l1.y),p2(a.l2.x,a.l2.y);
		
		for (list<line_seg>::iterator t=bound.begin();t!=bound.end();++t){ 
			if (intersect(p1,p2,pun(t->l1.x,t->l1.y),
					    pun(t->l2.x,t->l2.y),d)) {return false;}
		}
		FORALL(res,it) if (a.l2==it->first) { 
			FORALL(res,it2) if (it!=it2){
				d<?=(sint4)sqrt((double)distance2(it->first,it2->first))-6; 
			}
			break;
		}
		for (list<thick_point>::iterator t=res.begin();t!=res.end();++t){
			if (odl(p1,p2,pun(t->first.x,t->first.y))<d+t->second && a.l1!=t->first && a.l2!=t->first) {return false;}
		}
		return true;
}

list<Loc>& Graph::shortestPath(Loc p1, Loc p2){
//		cout << "dijkstra" <<endl;
		int v=closestFree(p1),w=closest(p2);
		double d[MAX_VERT];
		int pop[MAX_VERT];
		for (int i=0;i<MAX_VERT;++i) pop[i]=-2;
		pop[v]=-1;
		for (int i=0;i<MAX_VERT;++i) d[i]=1000000;
		d[v]=0;
		priority_queue<p_vert> q;
		q.push(p_vert(v,0));
		while (!q.empty()){
			int u=q.top().v;
			if (q.top().d>d[u]) { q.pop();continue;}
			q.pop();
			if (u==w) break;
			FOREACH(it,g[u])
				if (d[u]+v_dist(u,*it)<d[*it]){
					d[*it]=d[u]+v_dist(u,*it);
					q.push(p_vert(*it,d[*it]));
					pop[*it]=u;
				}
		}
		list<Loc> *r=new list<Loc>;
//		cout << "done"<<endl;
		for (int u=w;u!=-1;u=pop[u]) if (u==-2) {/* cout << "no path"<<endl;*/delete r; return *(new list<Loc>());} else
			r->push_front(Loc(v2coord[u].x, v2coord[u].y));
		if (p1!=v2coord[v])r->push_front(Loc(p1.x, p1.y));
		if (p2!=v2coord[w])r->push_back(Loc(p2.x, p2.y));
		return *r;
	}

list<Loc>& Graph::shortestPath(Loc p1,std::set<Loc>& endlocs){

		set<sint4> ends;
		FORALL(endlocs,it) {
			//cout << "end pos " << *it<<endl;  
			if (coord2v.count(*it)) 
			ends.insert(coord2v[*it]);
		}
		
		int v=closestFree(p1);
		double d[MAX_VERT];
		int pop[MAX_VERT];
		bool found_any=false;
		for (int i=0;i<MAX_VERT;++i) pop[i]=-2;
		pop[v]=-1;
		for (int i=0;i<MAX_VERT;++i) d[i]=1000000;
		d[v]=0;
		priority_queue<p_vert> q;
		q.push(p_vert(v,0));
		while (!q.empty()){
			int u=q.top().v;
			if (q.top().d>d[u]) { q.pop();continue;}
			q.pop();
			if (ends.count(u)){ found_any=true; break;}
			FOREACH(it,g[u])
				if (d[u]+v_dist(u,*it)<d[*it]){
					d[*it]=d[u]+v_dist(u,*it);
					q.push(p_vert(*it,d[*it]));
					pop[*it]=u;
				}
		}
		assert(found_any);
		list<Loc> *r=new list<Loc>;
		int w=-2;
		double mind=1000000;
		FORALL(ends,it){
			if (mind>d[(*it)]) {mind = d[(*it)];w = *it;}
		}
//		cout << "done"<<endl;
		for (int u=w;u!=-1;u=pop[u]) if (u==-2) {/* cout << "no path"<<endl;*/delete r; return *(new list<Loc>());} 
										else			r->push_front(v2coord[u]);
		r->push_front(Loc(p1.x, p1.y));
		return *r;
	}
	
	Graph::Graph(const Game &game, int _k, int _l, int d1, int  d2,
			list<line_seg> &_bound, list<thick_point>& _res):k(_k),l(_l),bound(_bound),res(_res){
		const Map<GameTile> &map=game.get_map();
		sint4 width=map.get_width()*game.get_tile_points(),
		    height=map.get_height()*game.get_tile_points();
		sint4 odx=width/(k+1),
		    ody=height/(l+1);
		int count=0;
		printf("%d %d\n",odx,ody);
		for (sint4 x=odx;x<width;x+=odx)
			for (sint4 y=ody;y<height;y+=ody){
				FORALL(res,it) if (distance2(it->first,Loc(x,y))<1000 && (it->first.x!=x || it->first.y!=y)&& testpath(line_seg(Loc(x,y),it->first),d2)){
					int v=getVertex(Loc(x,y)),w=getVertex(it->first);
					g[v].push_back(w);
		//			g[w].push_back(v);
					++count;
				}
				for (sint4 x1=x;x1<=min(width-odx,x+d1);x1+=odx)
					for (sint4 y1=max(y-d1,ody);y1<=min(height-ody,y+d1);y1+=ody){
						if ((x1!=x || y1!=y)&& testpath(line_seg(Loc(x,y),Loc(x1,y1)),d2)){
							int v=getVertex(Loc(x,y)),w=getVertex(Loc(x1,y1));
							g[v].push_back(w);
							g[w].push_back(v);
							++count;
						}
					}
			}
		
			
		cout << count << " krawedzi"<<endl;
		
		cout << " graph: made "<< count <<" edges"<<endl;
}

int Graph::closest(Loc p){
		sint4 d=100000,tmp;
		int v=0;
		for (int i=0;i<num_v;++i) if ((tmp=distance2(p,v2coord[i]))<d){ d=tmp; v=i;}
		return v;
}
int Graph::closestFree(Loc p){
		sint4 d=100000,tmp;
		int v=0;
		for (int i=0;i<num_v;++i){
			bool isfree=true;
			FORALL(Mineral::minerals,it) if ((*it)->loc==v2coord[i]) { isfree=false;break;} 
		 	if (isfree && (tmp=distance2(p,v2coord[i]))<d && testpath(line_seg(p,v2coord[i]),1)){ d=tmp; v=i;}
			}
		return v;
}	
	
double Graph::v_dist(int u, int v){
		return sqrt((double)distance2(v2coord[u],v2coord[v]));
}

GameObj* Graph::findClosest(GameObj& obj1){
	FORALL(Worker::w,it)
		if (Game::distance(obj1,*(*it)->obj)<1.5 && &obj1!=(*it)->obj) return (*it)->obj;
	FORALL(Sheep::sheeps,it)
		if (Game::distance(obj1,**it)<1.5) return *it;
	return NULL;
}
