// $Id: SampleEventHandler.C,v 1.9 2005/11/21 18:23:15 orts_mburo Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// A sample client for ORTS that shows how to access client data

// todo/questions:
//
// game scripts: 
// - what to do with workers during build time
// - similarly, how to define mining?
// - how is attack range defined (distance between centers/hulls)?
//
// client:
// - how to access global player data?
// - how to compute the set of visible tiles?
// - how to compute tiles visible from a unit?
// - how to decide what can be attacked by a specific unit?

// FORS/FORALL/ERR macros are defined in orts3/libs/kernel/src/Global.H

#include "SampleEventHandler.H"
#include "Worker.H"
#include "Graph.H"
#include "WorkerCoordinator.H"
#include "Mineral.H"
#include "AIState.H"

#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include "TerrainBase.H"

#include <assert.h>
#include <map>


using namespace std;

// tile types from core.bp
// fixme: game script reflection would help here
// in the competition we won't use water tiles
// TILE_CLIFF: non-traversable by ground units

enum { TILE_UNKNOWN=0, TILE_WATER=1, TILE_GROUND=2, TILE_CLIFF=3 };

bool SampleEventHandler::handle_event(const Event& e)
{
  if (e.get_who() == GameStateModule::FROM) {
    
    //cout << "Event: " << e.who() << " " << e.what() << endl;
    
    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    }
    
    if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    }
    
    if (e.get_what() == GameStateModule::VIEW_MSG) {
      
      compute_actions();
      state.gsm->send_actions();

      Game &game = state.gsm->get_game();	

      // gfx and frame statistics
      
      if (state.gfxm) state.gfxm->process_changes();
            if (state.gfxm) {
	      sint4 vf = game.get_view_frame();
      sint4 af = game.get_action_frame();
      //sint4 sa = game.get_skipped_actions();
      //sint4 ma = game.get_merged_actions();
	// don't draw if we are far behind
	if (abs(vf-af) < 10) {
	  state.gfxm->draw();
	  state.just_drew = true;
	}
      }
        
      return true;
    }
  }
    
  return false;
}

Graph& SampleEventHandler::constructGraph(const GameChanges& changes){
	static Game& game=state.gsm->get_game();
	static Graph* graph=NULL;
	if (!graph && !changes.new_boundaries.empty()){
		Mineral::tilepoints=game.get_tile_points();
	  cout << "buduje graf" << endl;
	  list<line_seg> b;
	  const Vector<ScriptObj*> &boundaries = changes.new_boundaries;
	  FORALL (boundaries, it) {
		  const GameObj *g = (*it)->get_GameObj();
		  if (g == 0) continue;
		  b.push_back(line_seg(Loc(*g->sod.x1,*g->sod.y1),
				Loc(*g->sod.x2,*g->sod.y2)));
	  }
	  const Game::ObjCont &neutral_objs = game.get_objs(game.get_player_num());
	  list<thick_point > c;
	  FORALL (neutral_objs, it) {
		  GameObj *gob = (*it)->get_GameObj();

		  if (gob==0) continue;
		  if (!gob->sod.in_game) continue;
//		  cout << "bp: " << gob->bp_name();

		  if(gob->bp_name()=="mineral"){
		//	cout << "przeszkoda " << *gob->sod.x << " " << *gob->sod.y << " " << *gob->sod.radius << endl;
			Mineral::addMineral(*gob);
			c.push_back(thick_point(Loc(*gob->sod.x,*gob->sod.y),*gob->sod.radius));
		  }else if (gob->bp_name()=="sheep") {
		  	Sheep::addSheep(*gob);
		  }
	  }
	  const Game::ObjCont &my_objs = game.get_objs(game.get_client_player());
	  FORALL (my_objs, it) {
		  GameObj *g = (*it)->get_GameObj();

		  if (g==0) continue;
		  if (!g->sod.in_game) continue;
//		  cout << "bp: " << gob->bp_name();
	//	  cout << "przeszkoda " << g->bp_name() <<endl;
		  if (g->bp_name()=="controlCenter"){
		  	Worker::setCommCenter(g);
			b.push_back(line_seg(Loc(*g->sod.x1,*g->sod.y1),
				Loc(*g->sod.x1,*g->sod.y2)));
			b.push_back(line_seg(Loc(*g->sod.x1,*g->sod.y1),
				Loc(*g->sod.x2,*g->sod.y1)));
			b.push_back(line_seg(Loc(*g->sod.x2,*g->sod.y2),
				Loc(*g->sod.x1,*g->sod.y2)));
			b.push_back(line_seg(Loc(*g->sod.x2,*g->sod.y2),
				Loc(*g->sod.x2,*g->sod.y1)));
		  }
	  }

		cout << "SEH: found "<<b.size()<<" bounds"<<endl;
	  graph=new Graph(game,30,30,32,9,b,c);
//	  Mineral::selectMinerals(512);
	  //graph=new Graph(game,30,30,32,6,b,c);
  }
  return *graph;
}
void SampleEventHandler::compute_actions()
{
//  cout << endl;


  const GameChanges &changes = state.gsm->get_changes();
 
  

  
  Graph& graph=constructGraph(changes);
  static WorkerCoordinator workerCoordinator(state,graph);
  workerCoordinator.setAction();


  


}
