#ifndef Worker_H
#define Worker_H

#include "Global.H"
#include "GameStateModule.H"
#include "TerrainBase.H"
#include <iostream>
#include <stdlib.h>

#include "GameObj.H"
#include "Graph.H"


using namespace std;
typedef TerrainBase::Loc Loc;

inline list<Loc>::iterator rev_iterator(list<Loc>& _list, list<Loc>::reverse_iterator rit){
	list<Loc>::iterator it=_list.begin();
	++rit;
	while (rit!=_list.rend()) {
		++rit; 
		++it;
	}
	return it;
}  
inline list<Loc>::reverse_iterator rev_iterator(list<Loc>& _list, list<Loc>::iterator it){
	list<Loc>::reverse_iterator rit=_list.rbegin();
	++it;
	while (it!=_list.end()) {
		++rit; 
		++it;
	}
	return rit;
}

class Worker {
	
	public:
	bool isWorking;
	Loc loc;
	Loc target; // waypoint
	Loc begin; // polozenie pocztkowe
	GameObj *obj;
	Graph& graph;
	GameObj *res;
	int id;
	int fetched;
	int turn;
	int staticAvoids;
	bool baseEncountered;
	GameStateModule& gsm;

	//enum State {NONE, FETCH, MINE, BACK, DROP, AVOID} state,previous;
	enum State {NONE, MOVE_TO, MINE, MOVE_BACK, UNLOAD, AVOID, EM_AVOID} state,previousState;
	sint4 avoidingTime;
	sint4 avoidCount;
	sint4 last_speed;
	list<Loc>* path;
	list<Loc>* emPath;
	list<Loc>::iterator emPathIt;
	list<Loc>::iterator pathIt;
	list<Loc>::reverse_iterator rPathIt;
	list<Loc>::reverse_iterator previousRPathIt;
	//list<Loc>::iterator current;
//	Worker(Graph& g,sint4 _x, sint4 _y, sint4 _tx, sint4 _ty):x(_x),y(_y),tx(_tx),ty(_ty),graph(g){}
	Worker(Graph& g, GameStateModule& _gsm,Loc _begin, int _id):
		isWorking(false),
		loc(_begin),
		target(Loc(-1,-1)), 
		begin(_begin), 
		graph(g),
		id(_id),
		fetched(0),
		turn(0),
		staticAvoids(0),
		baseEncountered(0),
		gsm(_gsm),
		state(NONE),
		avoidingTime(0),
		avoidCount(0),
		last_speed(0)
		{
	}

	

	bool reachedRes(){
		return (loc.x-*res->sod.x)*(loc.x-*res->sod.x)+(loc.y-*res->sod.y)*(loc.y-*res->sod.y)<100;
	}

	void setTarget(Loc newT){ 
		if (newT.x == 440 && loc.x<300) cout<<"worker "<<loc<<" "<<"setting target:"<<newT<<endl; 
		target=newT;}
	//void setTarget(Loc t){target.x=t.x;target.y=t.y;}
	
	bool hasTarget(){ return target.x!=-1;}
	bool targetReached() { return target==loc;}
	bool targetAlmostReached() { return (target.x-loc.x)*(target.x-loc.x)+(target.y-loc.y)*(target.y-loc.y)<100;}
	bool pathStepAlmostReached(){
		switch (state==AVOID ? previousState : state){
			case MOVE_TO:
				return (pathIt->x - loc.x)*(pathIt->x - loc.x) + (pathIt->y - loc.y)*(pathIt->y - loc.y)<512;
				break;
			case MOVE_BACK:
				return (rPathIt->x - loc.x)*(rPathIt->x - loc.x) + (rPathIt->y - loc.y)*(rPathIt->y - loc.y)<512;	
				break;
			case EM_AVOID:
				return (emPathIt->x - loc.x)*(emPathIt->x - loc.x) + (emPathIt->y - loc.y)*(emPathIt->y - loc.y)<512;
				break;
			default:
				ERR("setEndPathStep: default");
				break;
		}
	}

	void setResource(GameObj *r){res=r;}
	void setState(State s){
		//cout<<"Worker:changing state to:"<<s<<endl;
		if (state!=AVOID && state!=EM_AVOID)previousState=state;
		state=s;
		if (state==AVOID) avoidingTime=0;
	}
	void setPath(list<Loc>*);
	void setPath(Loc p);
	bool setEmPath(Loc p);
	void setPath(GameObj& gob);
	void setPathStep(){
		switch (state==AVOID ? previousState : state){
			case MOVE_TO:
				setTarget(*(pathIt));
				break;
			case MOVE_BACK:
				setTarget(*(rPathIt));	
				break;
			case EM_AVOID:
				setTarget(*(emPathIt));
				break;
			default:
				break;
		}
	}
	
	void setNextPathStep(){
		switch (state==AVOID ? previousState : state){
			case MOVE_TO:
				setTarget(*(++pathIt));
				break;
			case MOVE_BACK:
				setTarget(*(++rPathIt));	
				break;
			case EM_AVOID:
				setTarget(*(++emPathIt));
				break;
			default:
				break;
		}
	}
	void setEndPathStep(){
		switch (state==AVOID ? previousState : state){
			case MOVE_TO:
				setTarget(*(pathIt= --path->end()));
				break;
			case MOVE_BACK:
				setTarget(*(rPathIt= --path->rend()));	
				break;
			case EM_AVOID:
				setTarget(*(emPathIt= --emPath->end()));
				break;
			default:
				ERR("setEndPathStep: default");
				break;
		}
	}
	void reversePath(){rPathIt = path->rbegin(); setTarget(*(++rPathIt));}
	
	void normalizePath(){pathIt = path->begin(); setTarget(*(++pathIt));}
	
	void switchPath(){
		if (state==MOVE_BACK) {
			assert(-1);
			rPathIt=rev_iterator(*path, pathIt);
			setTarget(*(++rPathIt));
		}else{
			
			if (baseEncountered) pathIt=rev_iterator(*path, previousRPathIt);
			else	
			pathIt = path->begin(); 
			setTarget(*(++pathIt));
			baseEncountered = false;
		}
	}
	void emergencyPath(){emPathIt = emPath->begin(); setTarget(*emPathIt);}
	
	
	bool pathEnded(){
		switch(state==AVOID ? previousState : state){
			case MOVE_TO:
				return pathIt==--path->end();
			case MOVE_BACK:
				return rPathIt==--path->rend();
			case EM_AVOID:
				return emPathIt==--emPath->end();
			default: return 0;
		}
	} 
	bool hasPath(){	return path && !path->empty();	}
	
	bool hasState(){ return state!=NONE;}

	void dropResources(){
	}

	void onDoneJob();
	void gather();
	void gather(list<Loc>*);
	void avoid();
	void update();
	void setAction();
	void moveTo(Loc);
	void mine();
	void unload();
	void stop();
// do tworzenia i rozpoznawania workerow
	
	static vector<Worker*> w;
	const static sint4 step=15;
	static GameObj *commCenter;
	static Worker* getWorker(Graph& g, GameStateModule&gsm,GameObj *gob);
	static void setCommCenter(GameObj *cc){commCenter=cc;}
	
};

#endif /*Worker_H*/
